################################################################
## Script by T. Vu 
## Last updated 3/28/2010
################################################################
library(survival)

############ FUNCTIONS FOR T2E VPC PLOTS #######################
####################################################################################
## Function:	KMSurv
## Purpose:		compute KM probability of survival (no event) from simulated events
## Arguments:	eventFile - any files with only event times records       
## Returns:		probabilities of survival (no event) for simulated data
####################################################################################
KMSurv <- function(eventFile) {
   kmprob <- do.call(rbind,by(eventFile,list(eventFile$REP), function(subrep) {
      fit   <- survfit(Surv(TIME, DV)~1, data=subrep)
      out   <- summary(fit)
      surv  <- as.vector(out)$surv
      times <- as.vector(out)$time

      temp  <- cbind(TIME=times,SURV=surv)
      # linear interpolation
      temp2 <- approx(temp[,'TIME'],temp[,'SURV'],
                  xout=binTimes,
                  #seq(min(simFile$TIME),max(simFile$TIME),by=binTimes),
                  method='linear')
      maxt  <-  temp2$x[na.omit(temp2$y)==min(na.omit(temp2$y))]
      temp2$y[is.na(temp2$y)&temp2$x<min(temp[,'TIME'])] <- 1 #early time points outside of interpolation
      #temp2$y[temp2$x>maxt] <- temp2$y[temp2$x==maxt] #later time points outside of interpolation
      interp<- data.frame(REP=rep(unique(subrep$REP),times=length(temp2$x)),
                  TIME=temp2$x,SURV=temp2$y)
  }))
      return(kmprob)
}

KMSurvx <- function(eventFile) { #no interpolation
   kmprob <- do.call(rbind,by(eventFile,list(eventFile$REP), function(subrep) {
      fit   <- survfit(Surv(TIME, DV)~1, data=subrep)
      out   <- summary(fit)
      surv  <- as.vector(out)$surv
      times <- as.vector(out)$time

      temp  <- cbind(TIME=times,SURV=surv)
#      # linear interpolation
#      temp2 <- approx(temp[,'TIME'],temp[,'SURV'],
#                  xout=binTimes,
#                  #seq(min(simFile$TIME),max(simFile$TIME),by=binTimes),
#                  method='linear')
#      maxt  <-  temp2$x[na.omit(temp2$y)==min(na.omit(temp2$y))]
#      temp2$y[is.na(temp2$y)&temp2$x<min(temp[,'TIME'])] <- 1 #early time points outside of interpolation
#      temp2$y[temp2$x>maxt] <- temp2$y[temp2$x==maxt] #later time points outside of interpolation
      interp<- data.frame(REP=rep(unique(subrep$REP),times=length(temp[,'TIME'])),
                  TIME=temp[,'TIME'],SURV=temp[,'SURV'])
  }))
      return(kmprob)
}


####################################################################################
## Function:	KMSurvQuant
## Purpose:		compute quantiles of KM probability of survival (no event
## Arguments:	
## Returns:		probabilities of survival (no event) for simulated data
####################################################################################
KMSurvQuant <- function(KMSurvFile) {
   out <- do.call(rbind,by(KMSurvFile,list(KMSurvFile$TIME), function(subrep) {
      lowerp   <- (1-CIpercent)/2
      upperp   <- CIpercent+lowerp
      quant    <- quantile(subrep$SURV, probs=c(lowerp,.5,upperp), type=8, names=F)
      avg      <- mean(subrep$SURV)
      nobs     <- nrow(subrep)
      data.frame(TIME=unique(subrep$TIME),LOWPI=quant[1],N=nobs,MEAN=avg,MEDIAN=quant[2],
                  UPPERPI=quant[3])
   }))
   return(out)
}

KMSurvQuantx <- function(KMSurvFile) { #no interpolation
   out <- do.call(rbind,by(KMSurvFile,cut(KMSurvFile$TIME,br=binTimes,,right=F,include.lowest=T), function(subrep) {
      lowerp   <- (1-CIpercent)/2
      upperp   <- CIpercent+lowerp
      quant    <- quantile(subrep$SURV, probs=c(lowerp,.5,upperp), type=8, names=F)
      avg      <- mean(subrep$SURV)
      nobs     <- nrow(subrep)
      data.frame(TIME=median(subrep$TIME),LOWPI=quant[1],N=nobs,MEAN=avg,MEDIAN=quant[2],
                  UPPERPI=quant[3])
   }))
   return(out)
}

####################################################################################
## Function:	plotKM
## Purpose:		overlay observed with simulated events using KM method
## Arguments:	       
## Returns:		KM plot
####################################################################################
plotKM <- function(kmquantFile,obsFile) {
   ## Plot simulated events KM probabilities
   km    <- kmquantFile 
   km <- km[order(km$TIME),]
   if(!is.null(simxlim)) { km <- subset(km,TIME<=simxlim) }
   times <- km$TIME
   lowpi <- km$LOWPI
   hipi  <- km$UPPERPI
   medpi <- km$MEDIAN
   plot (times,medpi,type='n',
         cex.axis=1, #Does not seem to affect axis character size
         tcl=-.25, #tick length
         xlab="",ylab="",yaxt='n',xaxt='n',
         ylim=c(Ymin,1),xlim=c(0,Xmax))
   axis(side=1,las=0,at=seq(0,Xmax,Xtick),labels=paste(seq(0,Xmax,Xtick)), mgp=c(3,.5,0),tcl=-.25)
   axis(side=2,las=0,at=seq(Ymin,1,Ytick),labels=paste(seq(Ymin,1,Ytick)), mgp=c(3,.5,0),tcl=-.25)
   xx <-c(times,rev(times))
   yy <-c(lowpi[1:length(times)],rev(hipi[1:length(times)]))
   polygon(xx,yy, col=pCols['simpi'], border=NA,density=ifelse(pub,50,NA))
   lines(times,medpi,col=pCols['simmed'],lwd=2)

   ## Plot observed events
   event    <- subset(obsFile,select=c("ID","TIME","DV"))
   event    <- event[!is.na(event$DV),]
   #event$DV[event$DV==(-3)] <- 0 # convert censored events
   nevent   <- length(unique(event$ID[event$DV==1]))
   ncens    <- length(unique(event$ID[event$DV!=1]))

   obsfit   <- survfit(Surv(TIME,DV)~1, data=event,conf.type="plain",conf.int=CIpercent)

   lines(obsfit,conf.int=F,lwd=1.5,lty=1,col=pCols['obscol'],mark.time=FALSE)
   lines(obsfit,conf.int='only',lwd=1.5,lty=2,col=pCols['obscol'],mark.time=FALSE)
   
   mtext(side=1, line=2,paste("Time since study entry (",timeScaleTxt,")",sep=''),cex=1.5)
   mtext(side=2, line=2,paste(ylabel),cex=1.5)
   mtext(side=4,line=.5,cex=0.5,adj=.5,paste("obsFile:",obsFileName,"\n simFile:",modelName,sep=''))

   if(addLegend) {
      legend( 
          legend = c("Observed Kaplan-Meier","Predicted Median",
                     paste("Observed",CIpercent*100,"CI"),paste("Predicted",CIpercent*100,"CI")), 
          lty=c(1,1,2,1),lwd=c(rep(1.5,3),10),
          col=c(pCols['obscol'],pCols['simmed'],pCols['obscol'],pCols['simpi']),
          bty='n', cex=1,
          x=0.01*Xmax,y=Ymin+(1-Ymin)*0.35)
   }
}

####################################################################################
## Function:	readData
## Purpose:		read in simulated and observed data
## Arguments:	 
## Returns:	
####################################################################################
readData <- function(fileName) {
   return(read.csv(fileName,sep=',',header=T,na.string='.',comment.char=''))
}   
